"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InnerShipStatus = void 0;
const constant_1 = require("@skeldjs/constant");
const Networkable_1 = require("../Networkable");
const events_1 = require("../events");
class InnerShipStatus extends Networkable_1.Networkable {
    constructor(room, spawnType, netid, ownerid, flags, data) {
        super(room, spawnType, netid, ownerid, flags, data);
    }
    get owner() {
        return super.owner;
    }
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    Setup() { }
    Deserialize(reader, spawn = false) {
        if (!this.systems)
            this.Setup();
        while (reader.left) {
            const [tag, mreader] = reader.message();
            const system = this.systems[tag];
            if (system) {
                system.Deserialize(mreader, spawn);
            }
        }
    }
    /* eslint-disable-next-line */
    Serialize(writer, spawn = false) {
        if (!this.systems)
            this.Setup();
        const systems = Object.values(this.systems);
        for (let i = 0; i < systems.length; i++) {
            const system = systems[i];
            if (system === null || system === void 0 ? void 0 : system.dirty) {
                writer.begin(system.systemType);
                system.Serialize(writer, spawn);
                writer.end();
                system.dirty = false;
            }
        }
        this.dirtyBit = 0;
        return true;
    }
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.RepairSystem:
                await this._handleRepairSystem(rpc);
                break;
        }
    }
    FixedUpdate(delta) {
        if (!this.systems)
            this.Setup();
        const systems = Object.values(this.systems);
        for (let i = 0; i < systems.length; i++) {
            const system = systems[i];
            system === null || system === void 0 ? void 0 : system.Detoriorate(delta);
        }
    }
    async _handleRepairSystem(rpc) {
        const system = this.systems[rpc.systemid];
        const player = this.room.getPlayerByNetId(rpc.netid);
        if (system && player) {
            await system.HandleRepair(player, rpc.amount, rpc);
        }
    }
    /**
     * Randomly select players to be the Impostor. Called after a game is started
     * and emits a {@link RoomSelectImpostorsEvent} which can be used to alter the
     * results of this function.
     */
    async selectImpostors() {
        var _a, _b;
        const available = [...this.room.players.values()].filter((player) => player.info && !player.info.isDisconnected && !player.info.isDead);
        const max = available.length < 7 ? 1 : available.length < 9 ? 2 : 3;
        const impostors = [];
        for (let i = 0; i < Math.min(this.room.settings.numImpostors, max); i++) {
            const random = ~~(available.length - 1);
            impostors.push(available[random]);
            available.splice(random, 1);
        }
        const ev = await this.emit(new events_1.RoomSelectImpostorsEvent(this.room, impostors));
        if (!ev.canceled) {
            await ((_b = (_a = this.room.host) === null || _a === void 0 ? void 0 : _a.control) === null || _b === void 0 ? void 0 : _b.setImpostors(ev.alteredImpostors));
        }
    }
}
exports.InnerShipStatus = InnerShipStatus;
//# sourceMappingURL=InnerShipStatus.js.map